unit TestDateUtils;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testutils, testregistry;

type

  { TTestDateUtils }

  TTestDateUtils= class(TTestCase)
  private
  protected
  published
    procedure TestDecodeDate;
    procedure TestDecodeTime;
    procedure TestDateTimeToTimeStamp;
    procedure TestTimeStampToDateTime;
    procedure TestEncodeDate;
    procedure TestMSecsToTimeStamp;
  end;

implementation


{ TTestDateUtils }

procedure TTestDateUtils.TestDecodeDate;
var dt             : TDateTime;
    year,month,day : Word;
begin
  dt := 0;
  DecodeDate(dt,year,month,day);
  AssertEquals(1899,year);
  AssertEquals(12,month);
  AssertEquals(30,day);

  dt := 1;
  DecodeDate(dt,year,month,day);
  AssertEquals(1899,year);
  AssertEquals(12,month);
  AssertEquals(31,day);

  dt := -1;
  DecodeDate(dt,year,month,day);
  AssertEquals(1899,year);
  AssertEquals(12,month);
  AssertEquals(29,day);

  dt := -DateDelta;
  DecodeDate(dt,year,month,day);
  AssertEquals(0,year);
  AssertEquals(0,month);
  AssertEquals(0,day);

  dt := -DateDelta-5;
  DecodeDate(dt,year,month,day);
  AssertEquals(0,year);
  AssertEquals(0,month);
  AssertEquals(0,day);

  dt := -DateDelta+1;
  DecodeDate(dt,year,month,day);
  AssertEquals(1,year);
  AssertEquals(1,month);
  AssertEquals(1,day);

  dt := 2.75;
  DecodeDate(dt,year,month,day);
  AssertEquals(1900,year);
  AssertEquals(1,month);
  AssertEquals(1,day);

  dt := 2.25;
  DecodeDate(dt,year,month,day);
  AssertEquals(1900,year);
  AssertEquals(1,month);
  AssertEquals(1,day);

  dt := -1.25;
  DecodeDate(dt,year,month,day);
  AssertEquals(year,1899);
  AssertEquals(month,12);
  AssertEquals(day,29);

  dt := -1.75;
  DecodeDate(dt,year,month,day);
  AssertEquals(year,1899);
  AssertEquals(month,12);
  AssertEquals(day,29);
end;

procedure TTestDateUtils.TestDecodeTime;
var hour,minute,
    second,millisecond : Word;
begin
  DecodeTime(0,hour,minute,second,millisecond);
  AssertEquals(0,hour);
  AssertEquals(0,minute);
  AssertEquals(0,second);
  AssertEquals(0,millisecond);

  DecodeTime(4,hour,minute,second,millisecond);
  AssertEquals(0,hour);
  AssertEquals(0,minute);
  AssertEquals(0,second);
  AssertEquals(0,millisecond);

  DecodeTime(-4,hour,minute,second,millisecond);
  AssertEquals(0,hour);
  AssertEquals(0,minute);
  AssertEquals(0,second);
  AssertEquals(0,millisecond);

  DecodeTime(2.75,hour,minute,second,millisecond);
  AssertEquals(18,hour);
  AssertEquals(0,minute);
  AssertEquals(0,second);
  AssertEquals(0,millisecond);

  DecodeTime(-1.25,hour,minute,second,millisecond);
  AssertEquals(6,hour);
  AssertEquals(0,minute);
  AssertEquals(0,second);
  AssertEquals(0,millisecond);

  DecodeTime(0.5,hour,minute,second,millisecond);
  AssertEquals(12,hour);
  AssertEquals(0,minute);
  AssertEquals(0,second);
  AssertEquals(0,millisecond);
end;

procedure TTestDateUtils.TestDateTimeToTimeStamp;
var ts : TTimeStamp;
begin
  ts := DateTimeToTimeStamp(1);
  AssertEquals(DateDelta+1,ts.Date);
  AssertEquals(0,ts.Time);

  ts := DateTimeToTimeStamp(1.5);
  AssertEquals(DateDelta+1,ts.Date);
  AssertEquals(MSecsPerDay div 2,ts.Time);

  ts := DateTimeToTimeStamp(1.25);
  AssertEquals(DateDelta+1,ts.Date);
  AssertEquals(MSecsPerDay div 4,ts.Time);

  ts := DateTimeToTimeStamp(-1.25);
  AssertEquals(DateDelta-1,ts.Date);
  AssertEquals(MSecsPerDay div 4,ts.Time);

  ts := DateTimeToTimeStamp(-DateDelta+1);
  AssertEquals(1,ts.Date);
  AssertEquals(0,ts.Time);

end;

procedure TTestDateUtils.TestTimeStampToDateTime;
var ts : TTimeStamp;
    dt : TDateTime;
begin
  ts.Date := DateDelta+1;
  ts.time := 0;
  dt := TimeStampToDateTime(ts);
  AssertEquals(1,dt);

  ts.Date := DateDelta+1;
  ts.time := MSecsPerDay div 2;
  dt := TimeStampToDateTime(ts);
  AssertEquals(1.5,dt);

  ts.Date := DateDelta+1;
  ts.time := MSecsPerDay div 4;
  dt := TimeStampToDateTime(ts);
  AssertEquals(1.25,dt);

  ts.Date := DateDelta-1;
  ts.time := MSecsPerDay div 4;
  dt := TimeStampToDateTime(ts);
  AssertEquals(-1.25,dt);

  ts.Date := 1;
  ts.time := 0;
  dt := TimeStampToDateTime(ts);
  AssertEquals(-DateDelta+1,dt);

  ts.Date := 1;
  ts.time := MSecsPerDay div 2;
  dt := TimeStampToDateTime(ts);
  AssertEquals(-DateDelta+1-0.5,dt);

  ts.Date := DateDelta-1;
  ts.time := 3*(MSecsPerDay div 4);
  dt := TimeStampToDateTime(ts);
  AssertEquals(-1.75,dt);
end;

procedure TTestDateUtils.TestEncodeDate;
var dt             : TDateTime;
    year,month,day : Word;
    
begin
  year := 1899;
  month := 12;
  day := 30;
  dt := EncodeDate(year,month,day);
  AssertEquals(0,dt);

  year := 1899;
  month := 12;
  day := 31;
  dt := EncodeDate(year,month,day);
  AssertEquals(1,dt);

  year := 1899;
  month := 12;
  day := 29;
  dt := EncodeDate(year,month,day);
  AssertEquals(-1,dt);

{  year := 0;
  month := 0;
  day := 0;
  AssertException(EconvertError,EncodeDate(year,month,day));}

  year := 1;
  month := 1;
  day := 1;
  dt := EncodeDate(year,month,day);
  AssertEquals(-DateDelta+1,dt);

  year := 2004;
  month := 2;
  day := 29;
  dt := EncodeDate(year,month,day);
  AssertEquals(38046,dt);
end;

procedure TTestDateUtils.TestMSecsToTimeStamp;
var ts : TTimeStamp;

begin
  ts := MSecsToTimeStamp(1);
  AssertEquals(0,ts.Date);
  AssertEquals(1,ts.Time);

  ts := MSecsToTimeStamp(MSecsPerDay div 2);
  AssertEquals(0,ts.Date);
  AssertEquals(MSecsPerDay div 2,ts.Time);

  ts := MSecsToTimeStamp(MSecsPerDay * 3.75);
  AssertEquals(3,ts.Date);
  AssertEquals(3*(MSecsPerDay div 4),ts.Time);

{  ts := MSecsToTimeStamp(-1);
  AssertEquals(-1,ts.Date);
  AssertEquals(MSecsPerDay-1,ts.Time);}
end;


initialization
  RegisterTest(TTestDateUtils);
end.

